@@ -276,7 +276,7 @@ class GrlTrackerWrapper(GObject.GObject):
                     SELECT
                         %(media_type)s AS ?type
                         (COALESCE(?album_artist, ?artist) AS ?id)
-                        ?artist_bind AS ?albumArtist
+                        ?artist_bind AS ?artist
                     WHERE {
                         ?song a nmm:MusicPiece;
                                 nmm:musicAlbum ?album;
@@ -355,7 +355,7 @@ class GrlTrackerWrapper(GObject.GObject):
                     break
 
     def _song_media_query(self, media_ids):
-        media_ids = str(media_ids)[1:-1]
+        media_ids = str(media_ids)[1:-1].replace("'", "")
 
         query = """
         SELECT
@@ -599,14 +599,14 @@ class GrlTrackerWrapper(GObject.GObject):
                 artists_added.clear()
 
         query = """
-        SELECT ?type ?id ?albumArtist
+        SELECT ?type ?id ?artist
         WHERE {
             SERVICE <dbus:%(miner_fs_busname)s> {
                 GRAPH tracker:Audio {
                     SELECT
                        %(media_type)s AS ?type
                        (COALESCE(?album_artist, ?artist) AS ?id)
-                       ?artist_bind AS ?albumArtist
+                       ?artist_bind AS ?artist
                     WHERE {
                         ?song a nmm:MusicPiece;
                                 nmm:musicAlbum ?album;
