@@ -39,7 +39,7 @@ from gnomemusic.views.songsview import SongsView
 from gnomemusic.views.playlistsview import PlaylistsView
 from gnomemusic.widgets.headerbar import HeaderBar
 from gnomemusic.widgets.notificationspopup import NotificationsPopup  # noqa
-from gnomemusic.widgets.playertoolbar import PlayerToolbar
+from gnomemusic.widgets.playertoolbar import PlayerToolbar  # noqa: F401
 from gnomemusic.widgets.playlistdialog import PlaylistDialog
 from gnomemusic.widgets.searchheaderbar import SearchHeaderBar
 from gnomemusic.widgets.selectiontoolbar import SelectionToolbar  # noqa: F401
@@ -52,12 +52,13 @@ class Window(Gtk.ApplicationWindow):
     __gtype_name__ = "Window"
 
     active_view = GObject.Property(type=GObject.GObject, default=None)
-    selected_items_count = GObject.Property(type=int, default=0, minimum=0)
+    selected_songs_count = GObject.Property(type=int, default=0, minimum=0)
     selection_mode = GObject.Property(type=bool, default=False)
 
     notifications_popup = Gtk.Template.Child()
-    _box = Gtk.Template.Child()
+    _headerbar_stack = Gtk.Template.Child()
     _overlay = Gtk.Template.Child()
+    _player_toolbar = Gtk.Template.Child()
     _selection_toolbar = Gtk.Template.Child()
     _stack = Gtk.Template.Child()
 
@@ -68,11 +69,11 @@ class Window(Gtk.ApplicationWindow):
         """
         super().__init__(application=app, title=_("Music"))
 
-        # Hack
         self._app = app
+        self._coreselection = app.props.coreselection
 
-        self._app._coreselection.bind_property(
-            "selected-items-count", self, "selected-items-count")
+        self._coreselection.bind_property(
+            "selected-songs-count", self, "selected-songs-count")
 
         self._settings = app.props.settings
         self.add_action(self._settings.create_action('repeat'))
@@ -90,22 +91,20 @@ class Window(Gtk.ApplicationWindow):
         self._view_before_search = None
 
         self._player = app.props.player
+        self._search = app.props.search
 
+        self._startup_timeout_id = 0
         self._setup_view()
 
         MediaKeys(self._player, self)
 
     def _setup_view(self):
-        self._search = Search()
-        self._headerbar_stack = Gtk.Stack()
-        transition_type = Gtk.StackTransitionType.CROSSFADE
-        self._headerbar_stack.props.transition_type = transition_type
         self._headerbar = HeaderBar(self._app)
+        self._headerbar.props.stack = self._stack
         self._search_headerbar = SearchHeaderBar(self._app)
         self._search_headerbar.props.stack = self._stack
         self._headerbar_stack.add_named(self._headerbar, "main")
         self._headerbar_stack.add_named(self._search_headerbar, "search")
-        self._headerbar_stack.props.name = "search"
 
         self._search.bind_property(
             "search-mode-active", self._headerbar, "search-mode-active",
@@ -122,24 +121,23 @@ class Window(Gtk.ApplicationWindow):
         self._search.connect(
             "notify::search-mode-active", self._on_search_mode_changed)
 
-        self._player_toolbar = PlayerToolbar()
         self._player_toolbar.props.player = self._player
 
         self._headerbar.connect(
             'back-button-clicked', self._switch_back_from_childview)
 
         self.bind_property(
-            'selected-items-count', self._headerbar, 'selected-items-count')
+            'selected-songs-count', self._headerbar, 'selected-songs-count')
         self.bind_property(
-            "selected-items-count", self._selection_toolbar,
-            "selected-items-count")
+            "selected-songs-count", self._selection_toolbar,
+            "selected-songs-count")
         self.bind_property(
             'selection-mode', self._headerbar, 'selection-mode',
             GObject.BindingFlags.BIDIRECTIONAL
             | GObject.BindingFlags.SYNC_CREATE)
         self.bind_property(
-            "selected-items-count", self._search_headerbar,
-            "selected-items-count")
+            "selected-songs-count", self._search_headerbar,
+            "selected-songs-count")
         self.bind_property(
             "selection-mode", self._search_headerbar, "selection-mode",
             GObject.BindingFlags.BIDIRECTIONAL
@@ -151,7 +149,7 @@ class Window(Gtk.ApplicationWindow):
             "selection-mode", self._selection_toolbar, "visible")
         self.connect("notify::selection-mode", self._on_selection_mode_changed)
 
-        self.views = [Gtk.Box()] * len(View)
+        self.views = [None] * len(View)
         # Create only the empty view at startup
         # if no music, switch to empty view and hide stack
         # if some music is available, populate stack with mainviews,
@@ -165,33 +163,27 @@ class Window(Gtk.ApplicationWindow):
         # bottom line of the searchbar
         self._stack.get_style_context().add_class('background')
 
-        self._box.pack_end(self._player_toolbar, False, False, 0)
-
-        self.set_titlebar(self._headerbar_stack)
-
         self._selection_toolbar.connect(
             'add-to-playlist', self._on_add_to_playlist)
         self._search.connect("notify::state", self._on_search_state_changed)
 
         self._headerbar.props.state = HeaderBar.State.MAIN
-        self._headerbar_stack.show_all()
 
         self._app.props.coremodel.connect(
             "notify::songs-available", self._on_songs_available)
 
-        self._app.props.coremodel.props.grilo.connect(
+        self._app.props.coregrilo.connect(
             "notify::tracker-available", self._on_tracker_available)
 
-        if self._app.props.coremodel.props.songs_available:
-            self._switch_to_player_view()
-        else:
-            self._switch_to_empty_view()
+        self._startup_timeout_id = GLib.timeout_add(
+            2000, self._on_songs_available, None, None)
 
     def _switch_to_empty_view(self):
         did_initial_state = self._settings.get_boolean('did-initial-state')
 
-        state = self._app.props.coremodel.props.grilo.props.tracker_available
+        state = self._app.props.coregrilo.props.tracker_available
         empty_view = self.views[View.EMPTY]
+        empty_view.props.visible = True
         if state == TrackerState.UNAVAILABLE:
             empty_view.props.state = EmptyView.State.NO_TRACKER
         elif state == TrackerState.OUTDATED:
@@ -206,11 +198,15 @@ class Window(Gtk.ApplicationWindow):
 
     def _on_search_mode_changed(self, search, value):
         if self._search.props.search_mode_active:
-            self._headerbar_stack.set_visible_child_name("search")
+            self._headerbar_stack.props.visible_child_name = "search"
         else:
-            self._headerbar_stack.set_visible_child_name("main")
+            self._headerbar_stack.props.visible_child_name = "main"
 
     def _on_songs_available(self, klass, value):
+        if self._startup_timeout_id > 0:
+            GLib.source_remove(self._startup_timeout_id)
+            self._startup_timeout_id = 0
+
         if self._app.props.coremodel.props.songs_available:
             self._switch_to_player_view()
         else:
@@ -220,8 +216,7 @@ class Window(Gtk.ApplicationWindow):
         self.props.active_view = self._stack.props.visible_child
 
     def _on_tracker_available(self, klass, value):
-        grilo = self._app.props.coremodel.props.grilo
-        new_state = grilo.props.tracker_available
+        new_state = self._app.props.coregrilo.props.tracker_available
 
         if new_state != TrackerState.AVAILABLE:
             self._switch_to_empty_view()
@@ -244,21 +239,11 @@ class Window(Gtk.ApplicationWindow):
 
         self.views[View.EMPTY].props.state = EmptyView.State.SEARCH
 
-        # FIXME: In case Grilo is already initialized before the views
-        # get created, they never receive a 'ready' signal to trigger
-        # population. To fix this another check was added to baseview
-        # to populate if grilo is ready at the end of init. For this to
-        # work however, the headerbar stack needs to be created and
-        # populated. This is done below, by binding headerbar.stack to
-        # to window._stack. For this to succeed, the stack needs to be
-        # filled with something: Gtk.Box.
-        # This is a bit of circular logic that needs to be fixed.
         self._headerbar.props.state = HeaderBar.State.MAIN
-        self._headerbar.props.stack = self._stack
 
         # All views are created together, so if the album view is
         # already initialized, assume the rest are as well.
-        if not isinstance(self.views[View.ALBUM], Gtk.Box):
+        if self.views[View.ALBUM] is not None:
             return
 
         self.views[View.ALBUM] = AlbumsView(self._app)
@@ -272,12 +257,12 @@ class Window(Gtk.ApplicationWindow):
         # empty view state is changed once album view is visible to prevent it
         # from being displayed during startup
         for i in self.views[View.ALBUM:]:
-            if i.title:
-                self._stack.add_titled(i, i.name, i.title)
+            if i.props.title:
+                self._stack.add_titled(i, i.props.name, i.props.title)
             else:
-                self._stack.add_named(i, i.name)
+                self._stack.add_named(i, i.props.name)
 
-        self._stack.set_visible_child(self.views[View.ALBUM])
+        self._stack.props.visible_child = self.views[View.ALBUM]
 
         self.views[View.SEARCH].bind_property(
             "search-state", self._search, "state",
@@ -317,6 +302,7 @@ class Window(Gtk.ApplicationWindow):
             # Open search bar on Ctrl + F
             if (keyval == Gdk.KEY_f
                     and not self.views[View.PLAYLIST].rename_active
+                    and not self.props.selection_mode
                     and self._headerbar.props.state != HeaderBar.State.SEARCH):
                 search_mode = self._search.props.search_mode_active
                 self._search.props.search_mode_active = not search_mode
@@ -399,6 +385,7 @@ class Window(Gtk.ApplicationWindow):
                 and (modifiers == shift_mask
                      or modifiers == 0)
                 and not self.views[View.PLAYLIST].rename_active
+                and not self.props.selection_mode
                 and self._headerbar.props.state != HeaderBar.State.SEARCH):
             self._search.props.search_mode_active = True
 
@@ -449,7 +436,7 @@ class Window(Gtk.ApplicationWindow):
             return
 
         # Get back to the view before the search
-        self._stack.set_visible_child(self._view_before_search)
+        self._stack.props.visible_child = self._view_before_search
 
     def _switch_back_from_childview(self, klass=None):
         if self.props.selection_mode:
@@ -471,12 +458,13 @@ class Window(Gtk.ApplicationWindow):
         if self.props.active_view == self.views[View.PLAYLIST]:
             return
 
-        selected_songs = self._app._coreselection.props.selected_items
+        selected_songs = self._coreselection.props.selected_songs
 
         if len(selected_songs) < 1:
             return
 
-        playlist_dialog = PlaylistDialog(self)
+        playlist_dialog = PlaylistDialog(self._app)
+        playlist_dialog.props.transient_for = self
         if playlist_dialog.run() == Gtk.ResponseType.ACCEPT:
             playlist = playlist_dialog.props.selected_playlist
             playlist.add_songs(selected_songs)
